// 
// BoardAnalysisTests.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
namespace Draughts.Tests
{
	[TestFixture]
	public class BoardAnalysisTests
	{
		Board board;
		BoardAnalysis analysis;
		
		[SetUp]
		public void SetUp ()
		{
			
		}
		
		[Test]
		public void IsValidMove_WhiteForwardMovement_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "..w....." +
			            "........", Color.White);
						
			Assert.IsTrue (IsValidMove ("1-2 > 2-3"));
			Assert.IsTrue (IsValidMove ("1-2 > 2-1"));
		}
		
		[Test]
		public void IsValidMove_WhiteManBackwardMovement_ReturnsFalse ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "..w....." +
			            "........", Color.White);
						
			Assert.IsFalse (IsValidMove ("1-2 > 0-3"));
			Assert.IsFalse (IsValidMove ("1-2 > 0-1"));
		}
		
		[Test]
		public void IsValidMove_BlackManBackwardMovement_ReturnsFalse ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "..b....." +
			            "........", Color.Black);
						
			Assert.IsFalse (IsValidMove ("1-2 > 2-3"));
			Assert.IsFalse (IsValidMove ("1-2 > 2-1"));
		}
		
		[Test]
		public void IsValidMove_BlackManForwardMovement_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "..b....." +
			            "........", Color.Black);
						
			Assert.IsTrue (IsValidMove ("1-2 > 0-3"));
			Assert.IsTrue (IsValidMove ("1-2 > 0-1"));
		}
		
		[Test]
		public void IsValidMove_BlackManSingleEating_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            ".....b.." +
			            "....w..." +
			            "........" +
			            "........" +
			            "........", Color.Black);
						
			Assert.IsTrue (IsValidMove ("4-5 > 2-3"));
		}
		
		[Test]
		public void IsValidMove_WhiteManSingleEating_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            ".....b.." +
			            "....w..." +
			            "........" +
			            "........" +
			            "........", Color.White);
						
			Assert.IsTrue (IsValidMove ("3-4 > 5-6"));
		}
		
		[Test]
		public void IsValidMove_WhiteManMultipleEating_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            ".....b.." +
			            "........" +
			            ".....b.." +
			            "........" +
			            "...b...." +
			            "..w....." +
			            "........", Color.White);
						
			Assert.IsTrue (IsValidMove ("1-2 > 3-4 > 5-6 > 7-4"));
		}
		
		[Test]
		public void IsValidMove_WhiteKingMultipleEating_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "...b.b.." +
			            "........" +
			            ".B...b.." +
			            "........" +
			            "...b...." +
			            "..W....." +
			            "........", Color.White);
						
			Assert.IsTrue (IsValidMove ("1-2 > 3-4 > 5-6 > 7-4 > 5-2 > 3-0"));
		}
		
		[Test]
		public void IsValidMove_WhiteKingMultipleEatingSamePiece2Times_ReturnsFalse ()
		{
			GivenBoard ("........" +
			            "...b.b.." +
			            "........" +
			            "...b.b.." +
			            "........" +
			            "...b...." +
			            "..W....." +
			            "........", Color.White);
						
			Assert.IsFalse (IsValidMove ("1-2 > 3-4 > 5-6 > 7-4 > 5-2 > 3-4 > 5-6"));
		}
		
		[Test]
		public void IsValidMove_WhiteKingMultipleEatingGoingBackToSource_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "...b.b.." +
			            "........" +
			            "...b.b.." +
			            "....W..." +
			            "........" +
			            "........" +
			            "........", Color.White);
						
			Assert.IsTrue (IsValidMove ("3-4 > 5-6 > 7-4 > 5-2 > 3-4"));
		}
		
		[Test]
		public void IsValidMove_BlackKingMultipleEatingGoingBackToSource_ReturnsTrue ()
		{
			GivenBoard ("........" +
			            "...w.w.." +
			            "........" +
			            "...w.w.." +
			            "....B..." +
			            "........" +
			            "........" +
			            "........", Color.Black);
						
			Assert.IsTrue (IsValidMove ("3-4 > 5-6 > 7-4 > 5-2 > 3-4"));
		}
		
		[Test]
		public void GetMoveAction_OnMultipleEatingMove_Works ()
		{
			GivenBoard ("........" +
			            "...b.b.." +
			            "........" +
			            ".B...b.." +
			            "........" +
			            "...b...." +
			            "..W....." +
			            "........", Color.White);
						
			MoveDescription move = MoveBuilder.BuildMove ("1-2 > 3-4 > 5-6 > 7-4 > 5-2 > 3-0");
			IMoveAction action = analysis.GetMoveAction (move);
			action.Execute (board);
			
			Board expectedBoard = BoardBuilder.BuildSquareBoard (
			            "........" +
			            "........" +
			            "........" +
			            "........" +
			            "W......." +
			            "........" +
			            "........" +
			            "........");
			Assert.IsTrue (expectedBoard.Equals(board));
		}
		
		
		// Italian Move Costraints Tests Begin
		
		[Test]
		public void IsValidMove_BlackManMovingWhenForcedToEat_ReturnsFalse ()
		{
			GivenBoard ("........" +
			            "........" +
			            "....b..." +
			            "...w...." +
			            "........" +
			            "........" +
			            "........" +
			            "........", Color.Black);
						
			Assert.IsTrue (IsValidMove ("5-4 > 3-2"));
			Assert.IsFalse (IsValidMove ("5-4 > 4-5"));
		}
		
		[Test]
		public void IsValidMove_EatingSameNumberOfPieces_IsForcedToUseKing ()
		{
			GivenBoard ("........" +
			            "........" +
			            "..B.b..." +
			            "...w...." +
			            "........" +
			            "...w...." +
			            "........" +
			            "........", Color.Black);
						
			Assert.IsFalse (IsValidMove ("5-4 > 3-2 > 1-4"));
			Assert.IsTrue (IsValidMove ("5-2 > 3-4 > 1-2"));
		}
		
		[Test]
		public void IsValidMove_EatingSameNumberOfPieces_IsForcedToEatMoreKings ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "..b...B." +
			            "........" +
			            "..b...b." +
			            ".W...W..", Color.White);
						
			Assert.IsFalse (IsValidMove ("0-1 > 2-3 > 4-1"));
			Assert.IsTrue (IsValidMove ("0-5 > 2-7 > 4-5"));
		}
		
		[Test]
		public void IsValidMove_EatingSameNumberOfPieces_IsForcedToEatWhereKingOccursFirst ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "..b...B." +
			            "........" +
			            "..B...b." +
			            ".W...W..", Color.White);
						
			Assert.IsTrue (IsValidMove ("0-1 > 2-3 > 4-1"));
			Assert.IsFalse (IsValidMove ("0-5 > 2-7 > 4-5"));
		}
		
		[Test]
		public void IsValidMove_EatingSameKindOfPieces_CanChoose ()
		{
			GivenBoard ("........" +
			            "........" +
			            "........" +
			            "........" +
			            "..b...b." +
			            "........" +
			            "..b...b." +
			            ".W...W..", Color.White);
						
			Assert.IsTrue (IsValidMove ("0-1 > 2-3 > 4-1"));
			Assert.IsTrue (IsValidMove ("0-5 > 2-7 > 4-5"));
		}
		
		// Italian Move Costraints Tests End
		
		private void GivenBoard (string str, Color movingPlayer)
		{
			board = BoardBuilder.BuildSquareBoard (str);
			analysis = new BoardAnalysis (board, movingPlayer);
		}
		
		private bool IsValidMove (string move)
		{
			MoveDescription description = MoveBuilder.BuildMove (move);
			return analysis.IsValidMove (description);
		}
	}

	public static class MoveBuilder
	{
		public static MoveDescription BuildMove (string str)
		{
			List <BoardPosition> steps = new List <BoardPosition> ();
			
			foreach (string pos in str.Split ('>')) {
				string row = pos.Trim ().Split ('-')[0];
				string col = pos.Trim ().Split ('-')[1];
				
				BoardPosition position = new BoardPosition (int.Parse (row), int.Parse (col));
				steps.Add (position);
			}
			
			return new MoveDescription (steps);
		}
	}
}

