// 
// GLibEventLoop.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
namespace Draughts.UI
{
	// Summary: This is an utility class to remove repeated code.
	//          It allows to easily execute an action *once* without having to wrap the 
	//          action in a delegate returning false (which means "stop executing",
	//          returning true would have meant "execute me again")
	public static class GLibEventLoop
	{
		public static void ExecuteOnIdle (Action action)
		{
			GLib.Idle.Add (delegate {
				action ();
				return false;
			});
		}
		
		public static void ExecuteOnTimeout (uint milliseconds, Action action)
		{
			GLib.Timeout.Add (milliseconds, delegate {
				action ();
				return false;
			});
		}
		
		public static void ExecuteOnTimeout <T> (uint milliseconds, Action <T> action, T param)
		{
			GLib.Timeout.Add (milliseconds, delegate {
				action (param);
				return false;
			});
		}
	}
}

