// 
// NewGameDialog.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE

using System;
using Gtk;

namespace Draughts.UI
{
	// Summary: This dialog asks the user the kind of game he wants to start
	//          eg. Human vs Human, Human vs Computer - Hard ecc.
	//          To show the dialgo call the Run method.
	//          Output is given through the NewGameRequested event.
	public partial class NewGameDialog
	{
		public event Action <PlayerType, PlayerType> NewGameRequested;
		
		UserInterfaceCreator builder;
		Dialog dialog;
		
		public NewGameDialog ()
		{
			builder = new UserInterfaceCreator ("newgamedialog.glade");
			dialog = (Dialog) builder.GetObject ("dialog1");
			
			SetUpComboBoxes ();
		}
		
		public void Run ()
		{
			int response = dialog.Run ();
			
			// If "Start" was clicked, request a new game.
			if (response == 2) {
				PlayerType white = GetPlayerType ("whiteDifficultyCombo");
				PlayerType black = GetPlayerType ("blackDifficultyCombo");
				
				if (NewGameRequested != null)
					NewGameRequested (white, black);
			}
			
			dialog.Hide ();
		}
		
		PlayerType GetPlayerType (string widgetName)
		{
			ComboBox difficultyCombo = (ComboBox) builder.GetObject (widgetName);
			
			if (!difficultyCombo.Visible)
				return PlayerType.Human;
			
			else if (difficultyCombo.Active == 0)
				return PlayerType.VeryEasy;
			else if (difficultyCombo.Active == 1)
				return PlayerType.Easy;
			else if (difficultyCombo.Active == 2)
				return PlayerType.Medium;
			else if (difficultyCombo.Active == 3)
				return PlayerType.Hard;
			else if (difficultyCombo.Active == 4)
				return PlayerType.VeryHard;
			else 
				throw new InvalidOperationException ("Invalid combobox input");
		}
		
		void SetUpComboBoxes ()
		{
			ComboBox whitePlayerCombo = (ComboBox) builder.GetObject ("whitePlayerCombo");			
			ComboBox blackPlayerCombo = (ComboBox) builder.GetObject ("blackPlayerCombo");
			ComboBox whiteDifficultyCombo = (ComboBox) builder.GetObject ("whiteDifficultyCombo");
			ComboBox blackDifficultyCombo = (ComboBox) builder.GetObject ("blackDifficultyCombo");
			
			whitePlayerCombo.Active = 0; // White player is Human by default
			blackPlayerCombo.Active = 1; // while black is Computer
			whiteDifficultyCombo.Visible = false;
			blackDifficultyCombo.Visible = true;
			
			// Disable/Enable comboboxes when user change player type from "Human" to "Computer" 
			whitePlayerCombo.Changed += delegate {
				whiteDifficultyCombo.Visible = (whitePlayerCombo.Active == 1);
			};
			
			blackPlayerCombo.Changed += delegate {
				blackDifficultyCombo.Visible = (blackPlayerCombo.Active == 1);
			};
		}
	}
}
