// 
// UserInterfaceCreator.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using Gtk;
using System;
using System.Reflection;
using System.IO;

namespace Draughts.UI
{
	// To create Gtk user interfaces we use the Glade interface designer
	// together with the GtkBuilder class, which transforms .glade xml files
	// into user interfaces at runtime. 
	// Since we deploy .glade user interfaces as embedded resources in our
	// assembly, we need to extend GtkBuilder to read from them.
	public class UserInterfaceCreator : Builder
	{
		public UserInterfaceCreator (string resourceName)
		{
			Assembly assembly = Assembly.GetEntryAssembly ();
			
			using (Stream stream = assembly.GetManifestResourceStream (resourceName)) 
			using (TextReader reader = new StreamReader (stream))
			{
				string data = reader.ReadToEnd ();
				base.AddFromString (data);
			}
		}
	}
}
