// 
// BoardPosition.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Draughts
{
	// Summary: the BoardPosition struct identifies a location on a two dimensional board.
	//          Row/Column must be non negative.
	public struct BoardPosition : IEquatable <BoardPosition>
	{
		public readonly int Row;
		public readonly int Column;
		
		public BoardPosition (int row, int column)
		{
			// TODO (MONO 2.10): make use of code contracts to assure row/column are positive
			
			this.Row = row;
			this.Column = column;
		}
		
		public override string ToString ()
		{
			return string.Format ("[{0}-{1}]", Row, Column);
		}
		
		public static bool operator == (BoardPosition first, BoardPosition second)
		{
			return first.Equals (second);	
		}
		
		public static bool operator != (BoardPosition first, BoardPosition second)
		{
			return ! first.Equals (second);	
		}
		
		public override bool Equals (object obj)
		{
			if (obj is BoardPosition)
				return this.Equals ((BoardPosition) obj);
			else
				return false;
		}
		
		public bool Equals (BoardPosition other)
		{
			return this.Row == other.Row && this.Column == other.Column;
		}
		
		public override int GetHashCode ()
		{
			return Row * 89 + Column;
		}
	}
}

