// 
// ComputerPlayer.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Draughts
{
	// Summary: ComputerPlayer is a move provider which uses a MinMax algorithm to decide what to move.
	public class ComputerPlayer : IPlayer
	{
		MinMaxAlgorithm algorithm;
		Color color;
		Board board;
		
		public event Action <IPlayer, MoveDescription> MoveDecided;
		
		public ComputerPlayer (Color color, Board board, int recursionLevel, 
		                       EvaluationFuncion evaluationFunction)
		{
			this.color = color;
			this.board = board;
			algorithm = new MinMaxAlgorithm (recursionLevel, evaluationFunction);
		}
		
		public void StartGetMove ()
		{
			Console.WriteLine ("StartGetMove called on ComputerPlayer");
			MoveDescription move = algorithm.GetBestMove (board, color);
			MoveDecided (this, move);
		}
	}
}
