// 
// Draughts.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

namespace Draughts
{
	// Summary: Draghts enumumerates the different draughts pieces.
	//          Utility methods can be added through extension methods.
	public enum Draughts
	{
		BlackKing,
		BlackMan,
		WhiteKing,
		WhiteMan
	}
	
	public enum Color
	{
		Black,
		White
	}
	
	public enum Level
	{
		King,
		Man
	}
	
	public static class DraughtsExtensionMethodss
	{		
		public static Color GetColor (this Draughts piece)
		{
			if (piece == Draughts.BlackKing || piece == Draughts.BlackMan)
				return Color.Black;
			else
				return Color.White;
		}
		
		public static bool IsKing (this Draughts piece)
		{
			return piece == Draughts.BlackKing || piece == Draughts.WhiteKing;
		}
		
		public static bool IsMan (this Draughts piece)
		{
			return ! piece.IsKing ();
		}
		
		public static Level GetLevel (this Draughts piece)
		{
			if (piece.IsKing ())
				return Level.King;
			else
				return Level.Man;
		}
	}
	
	public static class ColorExtensionMethods
	{
		public static Color SwitchColor (this Color color)
		{
			if (color == Color.White)
				return Color.Black;
			else
				return Color.White;
		}
	}
}

