// 
// MoveDescription.cs
//  
// Author:
//       Matteo Nardi <matteo.nardi@gmx.com>
// 
// Copyright (c) 2011 Matteo Nardi
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Linq;
using System.Collections.Generic;

namespace Draughts
{
	// Summary: describes a draughts move with the list of board positions the moving piece jumps through.
	public class MoveDescription : IEquatable <MoveDescription>
	{
		public List <BoardPosition> Steps {
			get;
			private set;
		}
		
		public BoardPosition Source {
			get {
				return Steps.First ();
			}
		}
		
		public BoardPosition Destination {
			get {
				return Steps.Last ();
			}
		}
		
		public MoveDescription (List <BoardPosition> steps)
		{
			if (steps.Count < 2)
				throw new InvalidOperationException ("At least source and destination must di provided");
			
			this.Steps = new List <BoardPosition> (steps);
		}
		
		// Useful for non eating move
		public MoveDescription (BoardPosition source, BoardPosition destination)
		{
			this.Steps = new List<BoardPosition> {source, destination};
		}
		
		public bool Equals (MoveDescription other)
		{
			return this.Steps.SequenceEqual (other.Steps);
		}
		
		public override bool Equals (object obj)
		{
			if (obj is MoveDescription)
				return Equals ((MoveDescription) obj);
			else return false;
		}
		
		public override int GetHashCode ()
		{
			int hash = 0;
			foreach (BoardPosition position in Steps)
				hash = hash * 131 + position.GetHashCode ();
			
			return hash;
		}
	}
	
	static class MoveDescriptionUtils
	{
		public static bool IsEatingMove (this MoveDescription move)
		{
			return Math.Abs (move.Source.Row - move.Destination.Row) != 1;
		}
	}
}

